Package: nemsqar
Title: National Emergency Medical Service Quality Alliance Measure
        Calculations
Version: 1.2.1
Authors@R: c(
    person("Nicolas", "Foss", , "nicolas.foss@hhs.iowa.gov", role = c("aut", "cre")),
    person("Samuel", "Kordik", , "samuel.kordik@utsouthwestern.edu", role = "aut",
           comment = c(ORCID = "0000-0003-4230-1154")),
    person("Alyssa", "Green", , "agreen@nemsqa.org", role = "ctb"),
    person("Iowa Department of Health and Human Services", role = "cph")
  )
Description: Designed to automate the calculation of Emergency Medical
    Service (EMS) quality metrics, 'nemsqar' implements measures defined
    by the National EMS Quality Alliance (NEMSQA). By providing reliable,
    evidence-based quality assessments, the package supports EMS agencies,
    healthcare providers, and researchers in evaluating and improving
    patient outcomes.  Users can find details on all approved NEMSQA
    measures at <https://www.nemsqa.org/measures>.  Full technical
    specifications, including documentation and pseudocode used to develop
    'nemsqar', are available on the NEMSQA website after creating a user
    profile at <https://www.nemsqa.org>.
License: MIT + file LICENSE
URL: https://github.com/bemts-hhs/nemsqar,
        https://bemts-hhs.github.io/nemsqar/
BugReports: https://github.com/bemts-hhs/nemsqar/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.6.6), dplyr (>= 1.2.1), glue (>= 1.8.1), hms (>=
        1.1.4), lifecycle (>= 1.0.5), lubridate (>= 1.9.5), rlang (>=
        1.2.0), stats, tibble (>= 3.3.1), tidyselect (>= 1.2.1), utils
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0), tidyr
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
LazyDataCompression: bzip2
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-16 01:10:54 UTC; nfoss0
Author: Nicolas Foss [aut, cre],
  Samuel Kordik [aut] (ORCID: <https://orcid.org/0000-0003-4230-1154>),
  Alyssa Green [ctb],
  Iowa Department of Health and Human Services [cph]
Maintainer: Nicolas Foss <nicolas.foss@hhs.iowa.gov>
Repository: CRAN
Date/Publication: 2026-06-16 05:10:21 UTC
Built: R 4.5.2; ; 2026-06-16 06:41:16 UTC; unix
