# Generated by roxygen2: do not edit by hand

export(airway_01)
export(airway_01_population)
export(airway_05)
export(airway_05_population)
export(airway_18)
export(airway_18_population)
export(asthma_01)
export(asthma_01_population)
export(hypoglycemia_01)
export(hypoglycemia_01_population)
export(nemsqa_binomial_confint)
export(pediatrics_03b)
export(pediatrics_03b_population)
export(pretty_percent)
export(respiratory_01)
export(respiratory_01_population)
export(respiratory_02)
export(respiratory_02_population)
export(safety_01)
export(safety_01_population)
export(safety_02)
export(safety_02_population)
export(safety_04)
export(safety_04_population)
export(seizure_02)
export(seizure_02_population)
export(stroke_01)
export(stroke_01_population)
export(syncope_01)
export(syncope_01_population)
export(tbi_01)
export(tbi_01_population)
export(trauma_01)
export(trauma_01_population)
export(trauma_03)
export(trauma_03_population)
export(trauma_04)
export(trauma_04_population)
export(trauma_08)
export(trauma_08_population)
export(trauma_14)
export(trauma_14_population)
export(ttr_01)
export(ttr_01_population)
import(rlang)
importFrom(glue,glue)
importFrom(lifecycle,deprecated)
importFrom(stats,binom.test)
importFrom(stats,prop.test)
importFrom(utils,head)
