utils::globalVariables(c(
  "data_source",
  "column",
  "n_missing",
  "n",
  "prop_missing",
  "prop_missing_label",
  "procedures_datetime_name",
  "vitals_datetime_name",
  "airway_datetime_name",
  "airway_confirmation_col_name",
  "rr",
  "gcs",
  "dbp",
  "hr",
  "rr",
  "avpu",
  "AIRWAY_MANAGEMENT",
  "AIRWAY_PROCEDURE",
  "ALTERED",
  "ALTERED1",
  "ALTERED2",
  "AVPU",
  "BENZO_MED",
  "BLOOD_GLUCOSE",
  "CALL_911",
  "CARDIAC_ARREST",
  "CAR_SEAT",
  "CHEST",
  "DIABETES",
  "DIABETES1",
  "DIABETES2",
  "DOCUMENTED_WEIGHT",
  "DOCUMENTED_WEIGHT1",
  "DOCUMENTED_WEIGHT2",
  "ECG_PERFORMED",
  "ENDOTRACHEAL_INTUBATION",
  "EXTREMITIES",
  "FALL_HEIGHT",
  "GCS",
  "TRAUMA_CENTER",
  "HR_SBP_10_65_PLUS",
  "INJURY",
  "INJURY_CAUSE",
  "INTERFACILITY",
  "LAST_SUCCESSFUL_PROCEDURE",
  "LONG_BOARD",
  "LUNG",
  "MEDS_NOT_MISSING",
  "NEURO",
  "NON_WEIGHT_BASED",
  "NOT_CARDIAC_ARREST",
  "NOT_GCS_AVPU",
  "NO_LS_CHECK",
  "NO_TRANSPORT",
  "NUMERATOR",
  "NUMERATOR1",
  "NUMERATOR2",
  "OXYGEN",
  "OXYGEN1",
  "OXYGEN2",
  "PAIN_SCALE",
  "PAIN_SCALE_TIME",
  "PATIENT_CARE",
  "POSSIBLE_INJURY",
  "PROVIDER_IMPRESSION",
  "PULSE_OXIMETRY",
  "RESPIRATORY_DISTRESS",
  "RESPIRATORY_DISTRESS1",
  "RESPIRATORY_DISTRESS2",
  "RESPIRATORY_EFFORT",
  "RESPIRATORY_RATE_10",
  "SBP_10",
  "SCENE_DELAY",
  "SEIZURE",
  "SEVERE_INJURY",
  "STROKE",
  "STROKE1",
  "STROKE2",
  "STROKE_SCALE",
  "STROKE_SCALE1",
  "STROKE_SCALE2",
  "SYNCOPE",
  "SYNCOPE1",
  "SYNCOPE2",
  "TOURNIQUET",
  "TRANSPORT",
  "TRANSPORTS",
  "TRANSPORT_OR_INTERFACILITY",
  "TRAUMA_ALERT_10_64",
  "TRAUMA_ALERT_65",
  "TRAUMA_TRIAGE_1_2_10",
  "TRAUMA_TRIAGE_1_2_10_64",
  "TRAUMA_TRIAGE_1_2_65",
  "TRAUMA_TRIAGE_3_4",
  "TREATMENT",
  "TREATMENT1",
  "TREATMENT2",
  "VITALS",
  "VITALS_CHECK",
  "VITALS_CHECK1",
  "VITALS_CHECK2",
  "adult_measure_performance",
  "adult_population",
  "adult_successful_intubation",
  "adult_successful_no_hypotension",
  "adult_successful_no_hypoxia",
  "airway.05.population",
  "airway_table",
  "asthma",
  "beta_agonist_check",
  "bind_rows",
  "calc_age_10",
  "calc_age_10_64",
  "calc_age_65",
  "calc_age_adult",
  "calc_age_check",
  "calc_age_minor",
  "call_911",
  "cli_alert_success",
  "contains",
  "denominator",
  "distinct",
  "everything",
  "exclude_newborns",
  "exclude_pta_ca",
  "filter",
  "if_any",
  "include_population",
  "initial_pain_scale",
  "initial_pain_scale_time",
  "last_pain_scale",
  "last_pain_scale_time",
  "measure_performance",
  "mutate",
  "no_hypotension",
  "no_hypoxia",
  "numerator",
  "patient_age_in_days",
  "patient_age_in_days_col",
  "patient_age_in_years",
  "patient_age_in_years_col",
  "pedi_measure_performance",
  "pedi_population",
  "pedi_successful_intubation",
  "pedi_successful_no_hypotension",
  "pedi_successful_no_hypoxia",
  "procedure_order",
  "prop",
  "pull",
  "successful_no_hypotension",
  "successful_no_hypoxia",
  "successful_procedure",
  "system_age_10",
  "system_age_10_1",
  "system_age_10_2",
  "system_age_10_64",
  "system_age_65",
  "system_age_adult",
  "system_age_check",
  "system_age_check1",
  "system_age_check2",
  "system_age_minor",
  "system_age_minor1",
  "system_age_minor2",
  "system_age_minor3",
  "system_age_minor_exclusion",
  "system_age_minor_exclusion1",
  "system_age_minor_exclusion2",
  "vitals_range_end",
  "vitals_range_start",
  ":=",
  "CLEANED_AGE_UNITS",
  "INCIDENT_DATE",
  "across",
  "airway_01_vitals_table",
  "airway_05_vitals_table",
  "first_procedure",
  "first_successful_procedure",
  "matches",
  "non_missing_procedure_time",
  "not_performed_prior",
  "numerator1_10",
  "numerator1_1_28",
  "numerator1_1_9",
  "numerator1_all_sbp",
  "numerator1_all_spo2",
  "numerator2_10",
  "numerator2_1_28",
  "numerator2_1_9",
  "numerator2_all_sbp",
  "numerator2_all_spo2",
  "numerator3_1_28",
  "numerator4_1_28",
  "numerator_1",
  "numerator_10",
  "numerator_1_28",
  "numerator_1_9",
  "numerator_28",
  "numerator_all_sbp",
  "numerator_all_spo2",
  "patient_age_28_days_1_year",
  "patient_age_days_28",
  "patient_age_in_years_10_plus",
  "patient_age_in_years_1_9",
  "patient_age_in_years_multiplied",
  "range_bounds_after",
  "range_bounds_before",
  "successful_procedure_data",
  "target_procedures",
  "within_range_after",
  "within_range_before",
  "SBP_threshold",
  "airway_after_procedure",
  "airway_after_procedure_waveform",
  "first_attempt",
  "first_successful_attempt",
  "numerator1_sbp",
  "numerator2_sbp",
  "vitals_after_procedure",
  "vitals_after_procedure_waveform",
  "waveform_etc02_5",
  "waveform_etc02_used",
  "etc02",
  "sbp",
  "sp02",
  "vitals",
  "numerator_sbp_age"
))
