#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#' @importFrom glue glue
#' @importFrom lifecycle deprecated
#' @importFrom stats binom.test
#' @importFrom stats prop.test
#' @importFrom utils head
## usethis namespace: end
NULL
